/*

  Simple mains timer, Mark Atherton, 24 Jan 2010.

  60 min re-triggerable timer that is reset at power up.

  Active low Start Button has internal pull up enabled and is checked
  every 100ms. Output is active high and drives a solid state relay.

  5V is supplied by an dismantled phone changer and has 100nF decoupling
  cap between VCC and GND. Reset pin is tied high.

  Uses internal 9.6MHz RC clock, with /8 enabled. Heartbeat pin toggles at
  1Hz indicating that the timer is running.

*/

#include <tiny13.h>
#include <delay.h>

#define MIN(x)      (x*60)
#define nSTART	    PINB.4
#define RELAY	    PORTB.0
#define HEARTBEAT   PORTB.3
#define FALSE	    0
#define TRUE	    1

void main(void)
{
  unsigned int tim;
  unsigned char loop;

  #pragma optsize-
  CLKPR = 0x80;
  CLKPR = 0x00;
  #ifdef _OPTIMIZE_SIZE_
    #pragma optsize+
  #endif

  PORTB = 0x10; DDRB = 0x2F;            // PB4 in, all other out

  TCCR0A = 0x00; TCCR0B = 0x00;         // Timer disabled
  TCNT0 = 0x00;  OCR0A = 0x00;
  OCR0B = 0x00;

  GIMSK = 0x00;  MCUCR = 0x00;          // INT0: Off
  TIMSK0=0x00;

  ACSR=0x80;     ADCSRB=0x00;

  tim = 0;
  for(;;)
  {
    for(loop = 0; loop < 10; loop++)    // 1 sec loop
    {
      if(nSTART == 0) tim = MIN(60);    // retriggerable counter
      delay_ms(100);                    // 100ms response time
    }

    if(tim == 0)                        // terminal count ?
    {
      RELAY = FALSE;
    }
    else
    {
      RELAY = TRUE;
      HEARTBEAT ^= 1;
      tim--;                            // keep counting
    }
  }
}
